__all__ = ('Seen')


class Seen(object):
    """
    Container that returns ``False`` the first time a given object is
    tested for membership, and ``True`` thereafter.

    ``key`` should be a one-argument function used as a comparison key.

    Useful examples: ``Seen(hash)``, ``Seen(id)``

    >>> s = Seen(hash)
    >>> [x in s for x in (1, 2, 1, 2, 3)]
    [False, False, True, True, False]
    >>> s = Seen(id)
    >>> a, b, c = [[1], [2], [1]]
    >>> [x in s for x in (a, b, c, a, b, c)]
    [False, False, False, True, True, True]
    """
    def __init__(self, key):
        self._seen = set()
        self._key = key

    def __contains__(self, obj):
        obj_key = self._key(obj)
        if obj_key in self._seen:
            return True
        self._seen.add(obj_key)
        return False

    def __len__(self):
        return len(self._seen)

    def __repr__(self):
        return '<{} ({})>'.format(self.__class__.__name__, len(self))

    def clear(self):
        self._seen.clear()
